import { commands } from "../commands";
import { ConfiguratorSub } from "../configuratorOptions";
import { customCommandsActions } from "../customcmds/actionsDefinition";
import { Database } from "../db";
import { ActionForm, ModalForm } from "../form_func";
import { uiManager } from "../uis";
function clean(str) {
  return str.toLowerCase().replaceAll(' ', '-');
}
export function CUSTOM_COMMANDS() {
  let db = new Database("CustomCommands");
  commands.use((isCheckingCommandsList, cmdName, msg, args, theme, response, cmdsList, prefix) => {
    let commands = db.get("Commands", []);
    if (isCheckingCommandsList) {
      return commands.map(_ => {
        return {
          name: clean(_.name),
          description: _.description,
          category: _.category,
          aliases: [],
          admin: false,
          isDev: false,
          private: false,
          deprecated: false
        };
      });
    } else {
      if (!commands.find(_ => clean(_.name) == clean(cmdName))) return false;
      let cmdData = commands.find(_ => clean(_.name) == clean(cmdName));
      for (const action of cmdData.actions) {
        let actionData = customCommandsActions.find(_ => _.id == action.id);
        actionData.cb(...action.params.map(_ => {
          if (typeof _ !== "string") return;
          let toReturn = _;
          for (let i = 0; i < args.length; i++) {
            toReturn = toReturn.replaceAll(`$${i + 1}`, args[i]);
          }
          return toReturn;
        }), msg.sender, response);
      }
      return true;
    }
  });
  uiManager.addUI("Azalea1.0/CustomCommands/Add", player => {
    let modalForm = new ModalForm();
    modalForm.title("Add a command");
    modalForm.textField("Command name", "", undefined, (player, text, i) => {});
    modalForm.textField("Command description", "", undefined, (player, text, i) => {});
    modalForm.textField("Command category", "", undefined, (player, text, i) => {});
    modalForm.show(player, false, (player, response) => {
      let commands = db.get("Commands", []);
      let cmdName = response.formValues[0];
      let cmdDescription = response.formValues[1];
      let cmdCategory = response.formValues[2];
      commands.push({
        name: cmdName,
        description: cmdDescription,
        category: cmdCategory,
        author: player.name,
        actions: [],
        aliases: []
      });
      db.set("Commands", commands);
    });
  });
  uiManager.addUI("Azalea1.0/CustomCommands/Edit/AddAction/Configure", (player, actionID, name) => {
    let action = customCommandsActions.find(_ => _.id == actionID);
    let modalForm = new ModalForm();
    for (const actionParam of action.params) {
      if (actionParam.type == "text") {
        modalForm.textField(actionParam.label, actionParam.placeholder, undefined, () => {});
      }
    }
    modalForm.show(player, false, (player, response) => {
      let commands = db.get("Commands", []);
      let commandData = commands.find(_ => _.name == name);
      let commandIndex = commands.findIndex(_ => _.name == name);
      let params = [];
      for (const input of response.formValues) {
        params.push(input);
      }
      commandData.actions.push({
        params,
        id: actionID
      });
      commands[commandIndex] = commandData;
      db.set("Commands", commands);
      uiManager.open('Azalea1.0/CustomCommands/Edit', player, name);
    });
  });
  uiManager.addUI("Azalea1.0/CustomCommands/Edit/AddAction", (player, name) => {
    let commands = db.get("Commands", []);
    let commandData = commands.find(_ => _.name == name);
    let commandIndex = commands.findIndex(_ => _.name == name);
    let form = new ActionForm();
    for (const action of customCommandsActions) {
      form.button(action.ingameName, null, (player, i) => {
        uiManager.open("Azalea1.0/CustomCommands/Edit/AddAction/Configure", player, action.id, name);
      });
    }
    form.show(player, false, (player, response) => {});
  });
  uiManager.addUI("Azalea1.0/CustomCommands/EditAction", (player, name, actionIndex) => {
    let actionform = new ActionForm();
    actionform.button("Remove", null, (player, i) => {
      let commands = db.get("Commands", []);
      let commandData = commands.find(_ => _.name == name);
      let commandIndex = commands.findIndex(_ => _.name == name);
      commandData.actions.splice(actionIndex, 1);
      commands[commandIndex] = commandData;
      db.set("Commands", commands);
      uiManager.open('Azalea1.0/CustomCommands/Edit', player, name);
    });
    actionform.show(player, false, () => {});
  });
  uiManager.addUI("Azalea1.0/CustomCommands/Edit", (player, name) => {
    let commands = db.get("Commands", []);
    let commandData = commands.find(_ => _.name == name);
    let commandIndex = commands.findIndex(_ => _.name == name);
    let actionForm = new ActionForm();
    actionForm.button("Add Action", "azalea_icons/1", (player, i) => {
      uiManager.open("Azalea1.0/CustomCommands/Edit/AddAction", player, name);
    });
    let i = 0;
    for (const action of commandData.actions) {
      i++;
      actionForm.button(`ACTION ${i}`, null, (player, i2) => {
        uiManager.open("Azalea1.0/CustomCommands/EditAction", player, name, i - 1);
      });
    }
    actionForm.show(player, false, () => {});
  });
  uiManager.addUI("Azalea1.0/CustomCommands/Main", player => {
    let commands = db.get("Commands", []);
    if (!commands.length) {
      let form = new ActionForm();
      form.title("No commands");
      form.body("Imagine not creating commands");
      form.button("Create a command", null, (player, i) => {
        uiManager.open("Azalea1.0/CustomCommands/Add", player);
      });
      form.button("Exit", "azalea_icons/2");
      form.show(player, false, () => {});
      return;
    }
    let actionform = new ActionForm();
    actionform.button("Create a command", null, (player, i) => {
      uiManager.open("Azalea1.0/CustomCommands/Add", player);
    });
    for (const command of commands) {
      actionform.button(command.name, null, (player, i) => {
        uiManager.open("Azalea1.0/CustomCommands/Edit", player, command.name);
      });
    }
    actionform.show(player, false, (player, response) => {});
  });
  return new ConfiguratorSub("§9Custom Commands", "azalea_icons/CustomCommands").setCallback(player => {
    uiManager.open("Azalea1.0/CustomCommands/Main", player);
  });
}